# -*- coding: utf-8 -*-
# Generated by Django 1.11.27 on 2020-02-06 11:51


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0086_auto_20200128_1726'),
    ]

    operations = [
        migrations.AddField(
            model_name='historicalpendingenrollment',
            name='discount_percentage',
            field=models.DecimalField(decimal_places=5, default=0.0, max_digits=8),
        ),
        migrations.AddField(
            model_name='pendingenrollment',
            name='discount_percentage',
            field=models.DecimalField(decimal_places=5, default=0.0, max_digits=8),
        ),
    ]
