# Generated by Django 2.2.12 on 2020-05-07 11:38

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0094_add_use_enterprise_catalog_sample'),
    ]

    operations = [
        migrations.AddField(
            model_name='enterprisecustomer',
            name='enable_slug_login',
            field=models.BooleanField(default=False, help_text="Specifies whether the learner should be able to login through enterprise's slug login"),
        ),
        migrations.AddField(
            model_name='historicalenterprisecustomer',
            name='enable_slug_login',
            field=models.BooleanField(default=False, help_text="Specifies whether the learner should be able to login through enterprise's slug login"),
        ),
    ]
